// -----------------------------------------
// Rfid garden door code 
// 2018 - Hein Pragt
// -----------------------------------------

#include <SoftwareSerial.h>

SoftwareSerial rfid = SoftwareSerial(2, 3);

unsigned char *ids[] = {
"1D007D1FD8A",  // John
"1D007D1AD9A",  // Jenny
"1D008D1ED75",  // Benny
"1D007D1FDAA",  // Rick
"1D007D1EDCA",  // Anna
"1D007D1FD8A",  // Tom
"00000000000"
};

#define MAX_ID 6

#define LED_PIN   13
#define RELAY_PIN 12

int check_valid(void);
void open_door(void);
 
unsigned char buffer[13];
unsigned char index = 0;


void setup()  
{
  pinMode(RELAY_PIN, OUTPUT);
  pinMode(LED_PIN, OUTPUT);
  Serial.begin(9600);
  Serial.println("Serial Ready");
  rfid.begin(9600);
  Serial.println("RFID Ready");
}


void loop()
{
unsigned char c;
  
  while(rfid.available() > 0){
    c = rfid.read(); 
    if (c == 0x02) { 
       index = 0;
       continue;
    }   
    if (c == 0x03) { 
      Serial.println(" ");
      Serial.print("TAG: ");
      for (int i=0; i < 11; i++) { 
        Serial.write(buffer[i]);
      }
      if (check_valid() == 1) {
        Serial.print("VALID");
        open_door(); 
        while(rfid.available() > 0) c = rfid.read();  // Flush
      }
      index = 0;
      continue;
    }
    buffer[index] = c;
    if (index < 11) index++;
  }
}


// ----------------------------------
// Open the door
// ----------------------------------

void open_door() 
{
  digitalWrite(LED_PIN, HIGH);
  digitalWrite(RELAY_PIN, HIGH);
  delay(5000);
  digitalWrite(RELAY_PIN, LOW);  
  digitalWrite(LED_PIN, LOW);
}


// ----------------------------------
// Check for valied ID
// ----------------------------------

int check_valid() 
{
unsigned char *p1,*p2;
  
  for (int i=0; i < MAX_ID; i++) {
    p1 = ids[i];
    p2 = buffer;
    for (int j=0; j<11; j++) { 
      if (*p1++ != *p2++) {
        j = 12;  
      }
      else {
        if (j == 10) {
          return (1);
        }
      }
    }  
  }
  return(0);
}


